important_libraries = [
    'dask[complete]==2.11.0',
    'distributed==2.11.0',
    'cloudpickle==1.3.0',
    'ondemand-dask',
    'google-api-python-client',
    'jupyter-server-proxy',
    'google-cloud-storage',
    'bokeh',
    'google-cloud-bigquery',
    'gcsfs',
    'sqlalchemy',
    'dill',
]

extra_libraries = [
    'streamz',
    'confluent-kafka',
    'requests',
    'prometheus_client',
    'elasticsearch',
    'sentry-sdk',
    'influxdb',
    'pandas==0.25.3',
    'psycopg2-binary',
    'sklearn',
    'scikit-learn==0.22.1',
    'numpy',
    'scipy',
    'fuzzywuzzy[speedup]',
    'pendulum',
]
